#imports ABAQUS modules so we can get full functionality
from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import executeOnCaeStartup
#when CAE part of ABAQUS is loaded, execute this script
executeOnCaeStartup()

## auto-loading
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from job import *
from sketch import *
from visualization import *
## from connectorBehavior import *

#these are standard Python libraries added by people who modified this script
## my loadings
import math

#modifies settings to make the automatically-generated input file more readable
session.journalOptions.setValues(replayGeometry=COORDINATE,recoverGeometry=COORDINATE)

#more standard inputs
from decimal import *
from operator import *
from string import *

import datetime
import os
import random
import shutil
import time
import visualization

####################################################################
####################################################################
#create new model database, set variables


N_CPU= 2
MEM_ALLOWANCE = 1024*10 #10 gigs memory allowed for this job

########################################
### Modify these for new simulations ###
FILENAME = 'MG_20'
SUBNAME = 'ac_sq_new.f'
########################################

#Declarations for shape of part

a=1	#lattice constant
AA=0.5*a
BB=0.5*a
tt=0.025*a 
b=a-2.0*tt
#added by Joyce: customize how thick the channels are
vw=AA/10 #how thick the vertical channel is
hw=BB/10 #how thick the horizontal channel is

#constructor for model database
Mdb()



# Parameters

TOL = 0.001*a #tolerance 

CurModelName = 'Model-1'
CurJobName = FILENAME

#############################################
#############################################
numResult = 30
#resolution?
Reso = 0.05

### Create Part -- make initial shape
mdb.models['Model-1'].ConstrainedSketch(name='__profile__', sheetSize=60.0*a)
mdb.models['Model-1'].sketches['__profile__'].rectangle(point1=(-AA, -BB), 
    point2=(AA, BB))
mdb.models['Model-1'].Part(dimensionality=TWO_D_PLANAR, name='Part-1', type=
    DEFORMABLE_BODY)
mdb.models['Model-1'].parts['Part-1'].BaseShell(sketch=
    mdb.models['Model-1'].sketches['__profile__'])
del mdb.models['Model-1'].sketches['__profile__']
#make first cut
mdb.models['Model-1'].ConstrainedSketch(gridSpacing=0.0007, name='__profile__', 
    sheetSize=0.0282)
mdb.models['Model-1'].sketches['__profile__'].sketchOptions.setValues(
    decimalPlaces=4)
mdb.models['Model-1'].parts['Part-1'].projectReferencesOntoSketch(filter=
    COPLANAR_EDGES, sketch=mdb.models['Model-1'].sketches['__profile__'])
mdb.models['Model-1'].sketches['__profile__'].rectangle(point1=(-AA, BB), 
    point2=(-0.5*vw, 0.5*hw))
mdb.models['Model-1'].parts['Part-1'].Cut(sketch=
    mdb.models['Model-1'].sketches['__profile__'])
del mdb.models['Model-1'].sketches['__profile__']
#make second cut
mdb.models['Model-1'].ConstrainedSketch(gridSpacing=0.0007, name='__profile__', 
    sheetSize=0.0282)
mdb.models['Model-1'].sketches['__profile__'].sketchOptions.setValues(
    decimalPlaces=4)
mdb.models['Model-1'].parts['Part-1'].projectReferencesOntoSketch(filter=
    COPLANAR_EDGES, sketch=mdb.models['Model-1'].sketches['__profile__'])
mdb.models['Model-1'].sketches['__profile__'].rectangle(point1=(0.5*vw, 0.5*hw), 
    point2=(AA, BB))
mdb.models['Model-1'].parts['Part-1'].Cut(sketch=
    mdb.models['Model-1'].sketches['__profile__'])
del mdb.models['Model-1'].sketches['__profile__']
#make third cut
mdb.models['Model-1'].ConstrainedSketch(gridSpacing=0.0007, name='__profile__', 
    sheetSize=0.0282)
mdb.models['Model-1'].sketches['__profile__'].sketchOptions.setValues(
    decimalPlaces=4)
mdb.models['Model-1'].parts['Part-1'].projectReferencesOntoSketch(filter=
    COPLANAR_EDGES, sketch=mdb.models['Model-1'].sketches['__profile__'])
mdb.models['Model-1'].sketches['__profile__'].rectangle(point1=(-AA, -BB)
    , point2=(-0.5*vw, -0.5*hw))
mdb.models['Model-1'].parts['Part-1'].Cut(sketch=
    mdb.models['Model-1'].sketches['__profile__'])
del mdb.models['Model-1'].sketches['__profile__']
#make fourth cut
mdb.models['Model-1'].ConstrainedSketch(gridSpacing=0.0007, name='__profile__', 
    sheetSize=0.0282)
mdb.models['Model-1'].sketches['__profile__'].sketchOptions.setValues(
    decimalPlaces=4)
mdb.models['Model-1'].parts['Part-1'].projectReferencesOntoSketch(filter=
    COPLANAR_EDGES, sketch=mdb.models['Model-1'].sketches['__profile__'])
mdb.models['Model-1'].sketches['__profile__'].rectangle(point1=(0.5*vw, 
    -0.5*hw), point2=(AA, -BB))
mdb.models['Model-1'].parts['Part-1'].Cut(sketch=
    mdb.models['Model-1'].sketches['__profile__'])
del mdb.models['Model-1'].sketches['__profile__']

#test...make just one material called air, and the entire part is a section

mdb.models['Model-1'].Material(name='Air')
mdb.models['Model-1'].materials['Air'].Density(table=((1.2041, ), ))
mdb.models['Model-1'].materials['Air'].AcousticMedium(acousticVolumetricDrag=
    OFF, bulkTable=((141700.0, ), ))
mdb.models['Model-1'].HomogeneousSolidSection(material='Air', name=
    'Section-Air', thickness=None)
mdb.models['Model-1'].parts['Part-1'].SectionAssignment(offset=0.0, 
    offsetField='', offsetType=MIDDLE_SURFACE, region=Region(
    faces=mdb.models['Model-1'].parts['Part-1'].faces.findAt(((0.001833, 
    -8.3e-05, 0.0), (0.0, 0.0, 1.0)), )), sectionName='Section-Air', 
    thicknessAssignment=FROM_SECTION)

#######################################################
########### Fine Part and Solid Background for Viewing
mdb.models[CurModelName].parts['Part-1'].setValues(geometryRefinement=FINE)
session.graphicsOptions.setValues(backgroundStyle=SOLID)
#########################################################

### Assign Boundary Sets and Surfs (surfaces?)
#So 4 edges and the entire face?
mdb.models['Model-1'].parts['Part-1'].Set(edges=
    mdb.models['Model-1'].parts['Part-1'].edges.findAt(((0.0, BB, 0.0), 
    )), name='Set-top')
mdb.models['Model-1'].parts['Part-1'].Set(edges=
    mdb.models['Model-1'].parts['Part-1'].edges.findAt(((0.0, -BB, 0.0), 
    )), name='Set-bottom')
mdb.models['Model-1'].parts['Part-1'].Set(edges=
    mdb.models['Model-1'].parts['Part-1'].edges.findAt(((-AA, 0.0, 0.0), 
    )), name='Set-left')
mdb.models['Model-1'].parts['Part-1'].Set(edges=
    mdb.models['Model-1'].parts['Part-1'].edges.findAt(((AA, 0.0, 0.0), 
    )), name='Set-right')
mdb.models['Model-1'].parts['Part-1'].Set(faces=
    mdb.models['Model-1'].parts['Part-1'].faces.findAt(((0.0, 0.499*a, 0.0), 
    )), name='Set-body')	

#deleted unneeded partitioning
	
### Complete Part
##############################################################################

##deleted material, set, and section creation and assignment since it involved
##two different materials not just air
###############################################################################
#deleted partitioning for a finer mesh

###############################################################################

#### Mesh: Seeding -- revised because of cross shape.
# Copied from a GUI session -- just clicked Seed->Part and Mesh->Part
mdb.models['Model-1'].parts['Part-1'].seedPart(deviationFactor=0.1, 
    minSizeFactor=0.1, size=vw/4.0)

### Mesh: Element Type and Control
#I assume this is nothing to worry about? What about elemShape?
mdb.models['Model-1'].parts['Part-1'].setMeshControls(elemShape=QUAD, regions=
    mdb.models['Model-1'].parts['Part-1'].faces)

mdb.models['Model-1'].parts['Part-1'].setElementType(elemTypes=(ElemType(
    elemCode=AC2D4, elemLibrary=STANDARD), ElemType(elemCode=AC2D3, 
    elemLibrary=STANDARD)), regions=(
    mdb.models['Model-1'].parts['Part-1'].faces, ))

mdb.models['Model-1'].parts['Part-1'].generateMesh()


####################################################################################
####################################################################################

## instance kind of like object -- instantiating model/class
### Define Instance
mdb.models[CurModelName].rootAssembly.DatumCsysByDefault(CARTESIAN)

### Real Instance
mdb.models[CurModelName].rootAssembly.Instance(dependent=ON, name='Part-1-1', 
    part=mdb.models[CurModelName].parts['Part-1'])

### Imaginary Instance
mdb.models[CurModelName].rootAssembly.LinearInstancePattern(direction1=(1.0, 0.0, 
    0.0), direction2=(0.0, 1.0, 0.0), instanceList=('Part-1-1', ), number1=2, 
    number2=1, spacing1=2*2*AA, spacing2=2*BB)
mdb.models[CurModelName].rootAssembly.features.changeKey(fromName=
    'Part-1-1-lin-2-1', toName='Part-1-2')


########################################################################################
########################################################################################

### Start Assigning Nodes

### Real Nodes
a = mdb.models[CurModelName].rootAssembly

ALLNODES=a.instances['Part-1-1'].nodes
T_num1=[]
for NODES in ALLNODES:
      COORDS = NODES.coordinates
      if abs(COORDS[1]-BB)< TOL :
            T_num1.append(NODES.label)

B_num1=[]
for NODES in ALLNODES:
      COORDS = NODES.coordinates
      if abs(COORDS[1]+BB)< TOL :
            B_num1.append(NODES.label)

L_num1=[]
for NODES in ALLNODES:
      COORDS = NODES.coordinates
      if abs(COORDS[0]+AA)< TOL :
            L_num1.append(NODES.label)

R_num1=[]
for NODES in ALLNODES:
      COORDS = NODES.coordinates
      if abs(COORDS[0]-AA)< TOL :
            R_num1.append(NODES.label)

##
#The following 2 functions appear to just rearrange the order of the nodes based on some element
def REARRANGE0(ALLNODES,NODESET):
	TEMP=[]
	for NODELAB in NODESET:	
		NODE=ALLNODES.sequenceFromLabels((NODELAB,))
		TEMP.append(NODE[0].coordinates[0])
	TEMP.sort()
	REARRANGED=[0]*len(TEMP)
	for NODELAB in NODESET:
		NODE=ALLNODES.sequenceFromLabels((NODELAB,))
		REARRANGED[TEMP.index(NODE[0].coordinates[0])]=NODE[0].label
	return REARRANGED
##
##
def REARRANGE1(ALLNODES,NODESET):
	TEMP=[]
	for NODELAB in NODESET:	
		NODE=ALLNODES.sequenceFromLabels((NODELAB,))
		TEMP.append(NODE[0].coordinates[1])
        # print TEMP
	TEMP.sort()
        TEMP = sorted(TEMP)
        # print TEMP
	REARRANGED=[0]*len(TEMP)
	for NODELAB in NODESET:
		NODE=ALLNODES.sequenceFromLabels((NODELAB,))
		REARRANGED[TEMP.index(NODE[0].coordinates[1])]=NODE[0].label
	return REARRANGED
##
#print B_num1
B_num = REARRANGE0(ALLNODES,B_num1)
T_num = REARRANGE0(ALLNODES,T_num1)
R_num = REARRANGE1(ALLNODES,R_num1)
L_num = REARRANGE1(ALLNODES,L_num1)
#print B_num
# mdb.models[name].parts[name].SetFromNodeLabels

#create different sets of nodes?
B_set=a.SetFromNodeLabels(name='Bset' ,nodeLabels=(('Part-1-1',B_num ),('Part-1-2',[])))

T_set=a.SetFromNodeLabels(name='Tset' ,nodeLabels=(('Part-1-1',T_num ),('Part-1-2',[])))

R_set=a.SetFromNodeLabels(name='Rset' ,nodeLabels=(('Part-1-1',R_num ),('Part-1-2',[])))

L_set=a.SetFromNodeLabels(name='Lset' ,nodeLabels=(('Part-1-1',L_num ),('Part-1-2',[])))

# mdb.models[name].rootAssembly.SetFromElementLabels

# nodeLabels=(('Part-2-1', (2,3,5,7)), 
#    ('Instance-2', (1,2,3)))

# T_set=a.Set(name='Tset', nodes=a.instances['Part-2-1'].nodes.sequenceFromLabels(T_num),)

# R_set=a.Set(name='Rset', nodes=a.instances['Part-2-1'].nodes.sequenceFromLabels(R_num),)

# L_set=a.Set(name='Lset', nodes=a.instances['Part-2-1'].nodes.sequenceFromLabels(L_num),)


###########################
ll=0
ii=0
#looks like partitioning the nodes into different subgroups
for NODE in B_set.nodes:
	ll=ll+1		
        SETNAME='B_'+str(1000+ll)
	mdb.models[CurModelName].rootAssembly.Set(name=SETNAME, nodes=B_set.nodes[ii:ii+1]) 
	#must feed Set with a sequence
        ii = ii+1

ll=0
ii=0
for NODE in T_set.nodes:
	ll=ll+1		
        SETNAME='T_'+str(1000+ll)
	mdb.models[CurModelName].rootAssembly.Set(name=SETNAME, nodes=T_set.nodes[ii:ii+1]) 
	#must feed Set with a sequence
        ii = ii+1

ll=0
ii=0
for NODE in L_set.nodes:
	ll=ll+1		
        SETNAME='L_'+str(1000+ll)
	mdb.models[CurModelName].rootAssembly.Set(name=SETNAME, nodes=L_set.nodes[ii:ii+1]) 
	#must feed Set with a sequence
        ii = ii+1

ll=0
ii=0
for NODE in R_set.nodes:
	ll=ll+1		
        SETNAME='R_'+str(1000+ll)
	mdb.models[CurModelName].rootAssembly.Set(name=SETNAME, nodes=R_set.nodes[ii:ii+1]) 
	#must feed Set with a sequence
        ii = ii+1
		
##
###############################################################################


### Imaginary Nodes
a = mdb.models[CurModelName].rootAssembly

ALLNODES=a.instances['Part-1-2'].nodes
T_num2=[]
for NODES in ALLNODES:
      COORDS = NODES.coordinates
      if abs(COORDS[1]-BB)< TOL :
            T_num2.append(NODES.label)

B_num2=[]
for NODES in ALLNODES:
      COORDS = NODES.coordinates
      if abs(COORDS[1]+BB)< TOL :
            B_num2.append(NODES.label)

L_num2=[]
for NODES in ALLNODES:
      COORDS = NODES.coordinates
      if abs(COORDS[0]-3*AA)< TOL :
            L_num2.append(NODES.label)

R_num2=[]
for NODES in ALLNODES:
      COORDS = NODES.coordinates
      if abs(COORDS[0]-5*AA)< TOL :
            R_num2.append(NODES.label)


B_num = REARRANGE0(ALLNODES,B_num2)
T_num = REARRANGE0(ALLNODES,T_num2)
R_num = REARRANGE1(ALLNODES,R_num2)
L_num = REARRANGE1(ALLNODES,L_num2)


B2_set=a.SetFromNodeLabels(name='B2set' ,nodeLabels=(('Part-1-2',B_num ),('Part-1-1',[])))

T2_set=a.SetFromNodeLabels(name='T2set' ,nodeLabels=(('Part-1-2',T_num ),('Part-1-1',[])))

R2_set=a.SetFromNodeLabels(name='R2set' ,nodeLabels=(('Part-1-2',R_num ),('Part-1-1',[])))

L2_set=a.SetFromNodeLabels(name='L2set' ,nodeLabels=(('Part-1-2',L_num ),('Part-1-1',[])))

###########################
ll=0
ii=0
for NODE in B2_set.nodes:
	ll=ll+1		
        SETNAME='B2_'+str(1000+ll)
	mdb.models[CurModelName].rootAssembly.Set(name=SETNAME, nodes=B2_set.nodes[ii:ii+1]) 
	#must feed Set with a sequence
        ii = ii+1

ll=0
ii=0
for NODE in T2_set.nodes:
	ll=ll+1		
        SETNAME='T2_'+str(1000+ll)
	mdb.models[CurModelName].rootAssembly.Set(name=SETNAME, nodes=T2_set.nodes[ii:ii+1]) 
	#must feed Set with a sequence
        ii = ii+1

ll=0
ii=0
for NODE in L2_set.nodes:
	ll=ll+1		
        SETNAME='L2_'+str(1000+ll)
	mdb.models[CurModelName].rootAssembly.Set(name=SETNAME, nodes=L2_set.nodes[ii:ii+1]) 
	#must feed Set with a sequence
        ii = ii+1

ll=0
ii=0
for NODE in R2_set.nodes:
	ll=ll+1		
        SETNAME='R2_'+str(1000+ll)
	mdb.models[CurModelName].rootAssembly.Set(name=SETNAME, nodes=R2_set.nodes[ii:ii+1]) 
	#must feed Set with a sequence
        ii = ii+1
##

# 
#Assigning nodes in FAE means setting vertices so we can connect them
# Complete Assigning Nodes

########################################################################################
########################################################################################



###############################################################################
## Start of MPCs	
###############################################################################
##
##MPC = multi-point constraint
### MPCs for Wave Dispersion Analysis
##
ll=-1
for NODE in R_set.nodes:
	ll = ll+1
	SETNAME1='L_'+str(ll+1001)
	SETNAME2='L2_'+str(ll+1001)
	SETNAME3='R_'+str(ll+1001)
	
	mdb.models[CurModelName].MultipointConstraint(
	controlPoint=mdb.models[CurModelName].rootAssembly.sets[SETNAME1], 
	csys=None, 
	mpcType=USER_MPC, 
	name='Wave_'+SETNAME3, 
	surface=mdb.models[CurModelName].rootAssembly.sets[SETNAME3],
	userMode=NODE_MODE_MPC, 
	userType=1)

del mdb.models[CurModelName].constraints['Wave_'+SETNAME3] ### delete corner node constraint
	
ll=-1
for NODE in R2_set.nodes:
	ll = ll+1
	SETNAME1='L_'+str(ll+1001)
	SETNAME2='L2_'+str(ll+1001)
	SETNAME3='R2_'+str(ll+1001)

	mdb.models[CurModelName].MultipointConstraint(
	controlPoint=mdb.models[CurModelName].rootAssembly.sets[SETNAME1], 
	csys=None, 
	mpcType=USER_MPC, 
	name='Wave_'+SETNAME3, 
	surface=mdb.models[CurModelName].rootAssembly.sets[SETNAME3],
	userMode=NODE_MODE_MPC, 
	userType=2)

del mdb.models[CurModelName].constraints['Wave_'+SETNAME3] ### delete corner node constraint
	
##	
ll=-1
for NODE in T_set.nodes:
	ll = ll+1
	SETNAME1='B_'+str(ll+1001)
	SETNAME2='B2_'+str(ll+1001)
	SETNAME3='T_'+str(ll+1001)

	mdb.models[CurModelName].MultipointConstraint(
	controlPoint=mdb.models[CurModelName].rootAssembly.sets[SETNAME1], 
	csys=None, 
	mpcType=USER_MPC, 
	name='Wave_'+SETNAME3, 
	surface=mdb.models[CurModelName].rootAssembly.sets[SETNAME3],
	userMode=NODE_MODE_MPC, 
	userType=3)
######################################################################
### Corner Real
##### Delete corner node constraint
del mdb.models[CurModelName].constraints['Wave_'+SETNAME3] 
#### Create new MPC for Conner Node
SETNAME1 = 'B_1001'
mdb.models[CurModelName].MultipointConstraint(
	controlPoint=mdb.models[CurModelName].rootAssembly.sets[SETNAME1], 
	csys=None, 
	mpcType=USER_MPC, 
	name='Wave_'+SETNAME3, 
	surface=mdb.models[CurModelName].rootAssembly.sets[SETNAME3],
	userMode=NODE_MODE_MPC, 
	userType=5)


#######################################################################
	
ll=-1
for NODE in T2_set.nodes:
	ll = ll+1
	SETNAME1='B_'+str(ll+1001)
	SETNAME2='B2_'+str(ll+1001)
	SETNAME3='T2_'+str(ll+1001)

	mdb.models[CurModelName].MultipointConstraint(
	controlPoint=mdb.models[CurModelName].rootAssembly.sets[SETNAME1], 
	csys=None, 
	mpcType=USER_MPC, 
	name='Wave_'+SETNAME3, 
	surface=mdb.models[CurModelName].rootAssembly.sets[SETNAME3],
	userMode=NODE_MODE_MPC, 
	userType=4)

######################################################################
### Corner Imaginary
## Delete corner node constraint
del mdb.models[CurModelName].constraints['Wave_'+SETNAME3] 
######### Create new MPC for Conner Node
SETNAME1 = 'B_1001'
mdb.models[CurModelName].MultipointConstraint(
	controlPoint=mdb.models[CurModelName].rootAssembly.sets[SETNAME1], 
	csys=None, 
	mpcType=USER_MPC, 
	name='Wave_'+SETNAME3, 
	surface=mdb.models[CurModelName].rootAssembly.sets[SETNAME3],
	userMode=NODE_MODE_MPC, 
	userType=6)
#######################################################################
	
#####	
	
RightBottomNode = ll
	
mdb.models[CurModelName].rootAssembly.regenerate()


###############################################################################
###############################################################################
#so after this I guess nodes have been neatly assigned and constraints have been applied to everything
###############################################################################
### Define Step

### Deformation and Post-Buckling Step
CurJobName = FILENAME
mdb.models[CurModelName].FrequencyStep(name='step', numEigen=numResult, previous='Initial')


#########################
# Define the name of directories
wDir=os.getcwd()+'/' #working directory
# Create directories for individual results
rDir='Result'
if os.path.isdir(rDir)==False: #basically create result directory if necessary
    os.mkdir(rDir)
#########################

##
os.chdir(rDir) # Change to result directory
##

### Create job
mdb.Job(atTime=None, contactPrint=OFF, echoPrint=OFF, historyPrint=OFF, 
	model=CurModelName, 
	modelPrint=OFF, 
	name=CurJobName+'_tmp', 
	nodalOutputPrecision=FULL, 
    type=ANALYSIS, userSubroutine='',
	waitHours=0, waitMinutes=0)

### Write temporary input file	
mdb.jobs[CurJobName+'_tmp'].writeInput()	


InputFile=open(CurJobName+'.inp','w')
    
    # FLAG=1 for MPC-related line, FLAG=0 for others 
FLAG=0 
for line in open(CurJobName+'_tmp.inp'):
	if FLAG==1:
		### Static MPC
		if   line[0:2]=='11':       # For Static Analysis TYPE 11
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			InputFile.write(newline+', Virtual-1\n')
			
		elif   line[0:2]=='12':       # For Static Analysis TYPE 12
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			InputFile.write(newline+', Virtual-2\n')
		
		elif   line[0:2]=='13':       # For Static Analysis TYPE 13
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			InputFile.write(newline+', Virtual-1, Virtual-2\n')
		
		#######################################################
		### Wave MPC
		elif   line[0:2]=='1,':       # For Wave Analysis TYPE 1
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			node3 = line[-7] +'2'+line[-6:-1]
			InputFile.write(newline+', '+node3+'\n')
			
		elif   line[0:2]=='2,':       # For Wave Analysis TYPE 2
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			node3 = line[-7] +'2'+line[-6:-1]
			InputFile.write(newline+', '+node3+'\n')
			
		elif   line[0:2]=='3,':       # For Wave Analysis TYPE 3
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			node3 = line[-7] +'2'+line[-6:-1]
			InputFile.write(newline+', '+node3+'\n')
			
		elif   line[0:2]=='4,':       # For Wave Analysis TYPE 4
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			node3 = line[-7] +'2'+line[-6:-1]
			InputFile.write(newline+', '+node3+'\n')
			
		elif   line[0:2]=='5,':       # For Wave Analysis TYPE 5
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			node3 = line[-7] +'2'+line[-6:-1]
			InputFile.write(newline+', '+node3+'\n')
			
		elif   line[0:2]=='6,':       # For Wave Analysis TYPE 6
			#InputFile.write('*MPC, user, mode=node\n')
			newline = line[:-1] # Remove '\n' at the end !!!
			node3 = line[-7] +'2'+line[-6:-1]
			InputFile.write(newline+', '+node3+'\n')
	else:
		InputFile.write(line)
		
		
	if line[0:4]=='*MPC':
		FLAG=1      # FLAG=1 for MPC-related line
	else:
		FLAG=0      # FLAG=0 for others

InputFile.close()

mdb.JobFromInputFile(
		explicitPrecision=DOUBLE, 
		inputFileName=CurJobName+'.inp',
		name=FILENAME,
		nodalOutputPrecision=FULL,
		type=ANALYSIS, 
		memory = MEM_ALLOWANCE,
		memoryUnits = MEGA_BYTES, #MEGA_BYTES, and GIGA_BYTES. The default value is PERCENTAGE.
		numCpus=N_CPU, numDomains=N_CPU, 
		parallelizationMethodExplicit=DOMAIN, 
		userSubroutine=SUBNAME)

		
	
### Save and Submit
mdb.saveAs(pathName=FILENAME)
mdb.jobs[FILENAME].submit(consistencyChecking=OFF)
mdb.jobs[FILENAME].waitForCompletion()

### Clean Up after simulation! Appears to remove all the temporary files after the job is done
def clean(FILENAME):
	FileDelList=['.023','.mdl', '.sim', '.stt', '.com', '.fil', '.ipm', '.lck', '.prt', '_tmp.inp']
		#'.sta','.inp','.jnl','.msg','.odb']
	for i in FileDelList:
    		if os.path.exists(FILENAME+i)==True:
        		os.remove(FILENAME+i)
	return
	
# if Lambda != 0.9: # Save the Last one for Stress-Free Analysis
clean(FILENAME)

##
os.chdir(wDir) # Change back to python directory
## 

##################################################################################
##################################################################################
