#!/bin/bash

#Modification of files and creation of directories to prepare for mass ABAQUS job submission to SLURM.
#Current setup supports running a Python script in ABAQUS with a FORTRAN subroutine and using another
#Python script to perform postprocessing (pulling relevant data from a .dat file).

#Joyce Zhu 7/23/14 jqzhu@seas.harvard.edu
#If you'd like to do any complex modification of this script, please see the (long) README.
##################################
#Variables -- change by hand as needed for different simulations.
#Make sure they match up across all prepare, submit, run, and data collection scripts.

#Filenames for parallelization (a copy placed in each job's folder)
SCRIPT_FILE=Sq_new
SHELL_FILE=runAbaqus.sh
SUB_FILE=ac_sq_new.f
MATLAB_FILE_1=Read_Acoustic_Data.m
MATLAB_FILE_2=Plot_Acoustic.m
MATLAB_FILE_3=ftn_figureSet.m
#remember to go into your DAT_READER, or postprocessing, file to set your final output file 
#name--this is more efficient than having the shell change it to the same name in every copied file
DAT_READER=readAbaqusParallel.py

#These are job-specific values -- variables you are modifying or copying into every copy of a specific
#file in a job directory as well as variables to increment values and set job names. Add, modify, or delete 
#anything (the NAMES and NUMBER_STEPS variables are needed for incrementing though)
RESOLUTION=0.05
KX=0
KY=0
K_STEP_SIZE=1
NUMBER_STEPS=3
NAMES=([1]="GX" [2]="XM" [3]="MG")

#locations of where you want to execute your work (scratch) and where you want to store final results (group)
SCRATCH_DIR=/n/home10/jqzhu/slurm
GROUP_DIR=/n/home10/jqzhu/abaqusResults
#also change this formula for calculating how many jobs there are per step if needed
JOBS_PER_STEP=$(python -c "print int(1.0 / $RESOLUTION)")
#these two are used for incrementing; do not change
CURRENT_JOB=1
let "TOTAL_JOBS=$NUMBER_STEPS*$JOBS_PER_STEP"
##################################
STEP_NAME=dummyname
for x in `seq 1 $NUMBER_STEPS`; 
do
   for i in `seq 1 $JOBS_PER_STEP`;
   do
      #first, replace variable values using the UNIX stream editor. In this case, we replace
      #Kx, Ky, and the frequency step name in the ABAQUS script and subroutine file and update filenames
      #in the MATLAB postprocessing files and the Python output reader script
      sed -i s/^"\s\{2,\}Kx =".*/"          Kx = $KX"/ $SUB_FILE
      sed -i s/^"\s\{2,\}Ky =".*/"          Ky = $KY"/ $SUB_FILE
      #An example step name in this case would be "MG_2"
      STEP_NAME="${NAMES[$x]}"_$i
      sed -i s/"FILENAME =".*/"FILENAME = \'$STEP_NAME\'"/ $SCRIPT_FILE.py
      sed -i s/"DIR_NAME=".*/"DIR_NAME=$STEP_NAME"/ $SHELL_FILE
      sed -i s/^"\s\{2,\}FILENAME =".*/"    FILENAME = ['$STEP_NAME']"/ $MATLAB_FILE_2
      sed -i s/^"\s\{2,\}FILENAME =".*/"    FILENAME = ['$STEP_NAME']"/ $MATLAB_FILE_1
      sed -i s/"DAT_FILENAME =".*/"DAT_FILENAME = \'$STEP_NAME.dat\'"/ $DAT_READER
      sed -i s/"STEP_NUMBER =".*/"STEP_NUMBER = $CURRENT_JOB"/ $DAT_READER
      #make a directory for each job and a Result subdirectory and copy everything over
      mkdir $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE
      mkdir $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE/Result
      cp $SCRIPT_FILE.py $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE
      cp $SUB_FILE $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE/Result
      cp $SHELL_FILE $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE
      cp $MATLAB_FILE_1 $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE/Result
      cp $MATLAB_FILE_2 $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE/Result
      cp $MATLAB_FILE_3 $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE/Result
      cp $DAT_READER $SCRATCH_DIR/job_"$CURRENT_JOB"-"$STEP_NAME"_$SCRIPT_FILE/Result
      #update necessary variables
      let "CURRENT_JOB=CURRENT_JOB+1"
      KY=$(python -c "print $KY + $K_STEP_SIZE")
      KX=$(python -c "print $KX + $K_STEP_SIZE") 
   done
done
echo
echo "File and folder modification and creation complete. Please use job submission script now."
