#!/bin/bash
#
#SLURM submission job options
#SBATCH -n 8                 # Number of cores requested MAXIMUM 16 FOR ABAQUS!!!
#SBATCH -N 5                 # Total number of computers/nodes
#SBATCH -p general           # Partition to submit to
#SBATCH --mem-per-cpu=700  # Memory per cpu in MB (see also --mem)
#SBATCH -o out               # Standard out goes to this file
#SBATCH -e err               # Standard err goes to this file
#
#=====================================================================
# This script is for running ABAQUS with the following conditions:
# 
# [S]  SMP parallelism
# [S]  SLURM job management 
# 
#Joyce Zhu 6/20/14 jqzhu@seas.harvard.edu
#Modified 7/10/14 to support SLURM instead of SGE
#=====================================================================

pwd

##################################
#Again, MAKE SURE these shared variables have values matching the ones in the previous
#two shell scripts.
SCRIPT_FILE=Sq_new.py
SUB_FILE=ac_sq_new.f
DAT_READER=readAbaqusParallel.py
JOBNAME=$SLURM_JOBID
SCRATCH_DIR=/n/home10/jqzhu
GROUP_DIR=/n/home10/jqzhu/abaqusResults
#don't change this by hand, that's what the prepare shell script automates!
DIR_NAME=MG_20
##################################

#--------------------------------------------------------------------------------------------
# Load modules: ABAQUS version 6.11 and the Intel FORTRAN compiler (and gcc for good measure)
#--------------------------------------------------------------------------------------------
module load centos6/openmpi-1.6.5_gcc-4.8.0
module load seas/abaqus-6.11-1
module load centos6/openmpi-1.4.2_intel-11.1.072

#handle unexpected job termination
exit_gracefully () {
  abaqus terminate job=$JOBNAME
  echo Abaqus job $JOBNAME terminated
  exit
}
 
trap exit_gracefully SIGUSR2
date

#if you have .inp file, instead of noGUI option, use input=$INPUT_FILENAME
abaqus cae noGUI=$SCRIPT_FILE
date

#----------------------------------------------------------------------- 
# Prints out machine details for debugging, if needed
#----------------------------------------------------------------------- 
/bin/uname -a
 
#----------------------------------------------------------------------- 
# Script pauses until ABAQUS is done executing (lock file disappears)
#----------------------------------------------------------------------- 
sleep 60
while [ -f Result/*.lck ]; do
  sleep 5
done
exit
#----------------------------------------------------------------------- 
# Copy important files into group directory
#-----------------------------------------------------------------------
mkdir $GROUP_DIR/"$DIR_NAME"
cd Result
cp *.dat $GROUP_DIR/"$DIR_NAME"
cp *.odb $GROUP_DIR/"$DIR_NAME"

#Call post-processing script to collect relevant data
python $DAT_READER

exit
